"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** */
function createEntityFactory() {
    return function createEntity(key) {
        return { key: key, ids: [], all: {} };
    };
}
exports.createEntityFactory = createEntityFactory;
/**
 * Inserts values into an array,
 * skipping values that already exists.
 *
 * @param array Array to insert into.
 * @param values Values to insert.
 *
 * @example
 * const numbers = []
 * insertIntoArray(numbers, 1, 2, 3, 4, 5)
 * // numbers = [1, 2, 3, 4, 5]
 */
function insertIntoArray(array) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    var exists = new Set(array);
    for (var _a = 0, values_1 = values; _a < values_1.length; _a++) {
        var value = values_1[_a];
        if (!exists.has(value)) {
            exists.add(value);
            array.push(value);
        }
    }
}
exports.insertIntoArray = insertIntoArray;
/**
 * Removes values from an array.
 *
 * @param array Array to remove from.
 * @param values Values to remove.
 *
 * @example
 * const numbers = [0, 1, 2, 3, 4, 5]
 * removeFromArray(numbers, 0, 2, 4)
 * // numbers = [1, 3, 5]
 */
function removeFromArray(array) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    for (var _a = 0, values_2 = values; _a < values_2.length; _a++) {
        var value = values_2[_a];
        var index = array.indexOf(value);
        index < 0 || array.splice(index, 1);
    }
}
exports.removeFromArray = removeFromArray;
/**
 * Removes all values from an entity.
 * Does not remove the key value.
 *
 * @param entity Entity to clear.
 *
 * @example
 * clearEntity(people)
 */
function clearEntity(entity) {
    for (var _i = 0, _a = entity.ids; _i < _a.length; _i++) {
        var id = _a[_i];
        delete entity.all[id];
    }
    entity.ids.length = 0;
}
exports.clearEntity = clearEntity;
/**
 * Copies an entity with all its values.
 * The values themselves are not copied,
 * so they keep their referential equality.
 *
 * @param entity Entity to copy.
 *
 * @example
 * copyEntity(people)
 */
function copyEntity(entity) {
    var copy = createEntityFactory()(entity.key);
    for (var _i = 0, _a = entity.ids; _i < _a.length; _i++) {
        var id = _a[_i];
        copy.ids.push(id);
        copy.all[id] = entity.all[id];
    }
    return copy;
}
exports.copyEntity = copyEntity;
/**
 * Inserts values into an entity.
 *
 * @param entity Entity to insert into.
 * @param values Values to insert.
 *
 * @example
 * insertIntoEntity(people, matt, sam, liam)
 */
function insertIntoEntity(entity) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    var exists = new Set(entity.ids);
    for (var _a = 0, values_3 = values; _a < values_3.length; _a++) {
        var value = values_3[_a];
        var id = value[entity.key];
        entity.all[id] = value;
        if (!exists.has(id)) {
            exists.add(id);
            entity.ids.push(id);
        }
    }
}
exports.insertIntoEntity = insertIntoEntity;
/**
 * Removes values from an entity.
 *
 * @param entity Entity to remove from.
 * @param values Values to remove.
 *
 * @example
 * removeFromEntity(people, matt, sam, liam)
 */
function removeFromEntity(entity) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    for (var _a = 0, values_4 = values; _a < values_4.length; _a++) {
        var value = values_4[_a];
        var id = value[entity.key];
        delete entity.all[id];
        removeFromArray(entity.ids, id);
    }
}
exports.removeFromEntity = removeFromEntity;
/**
 * Removes values from an entity by matching IDs.
 *
 * @param entity Entity to remove from.
 * @param ids IDs to match.
 *
 * @example
 * removeFromEntityById(people, 0, 2, 6)
 */
function removeFromEntityById(entity) {
    var ids = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        ids[_i - 1] = arguments[_i];
    }
    for (var _a = 0, ids_1 = ids; _a < ids_1.length; _a++) {
        var id = ids_1[_a];
        delete entity.all[id];
        removeFromArray(entity.ids, id);
    }
}
exports.removeFromEntityById = removeFromEntityById;
