import { Entity, KeyType, Value, ValueType } from './type-helpers';
export { Entity };
/** */
export declare function createEntityFactory<T extends Value>(): <K extends { [K in keyof T]: T[K] extends string | number | symbol ? K : never; }[keyof T]>(key: K) => Entity<T, K>;
/**
 * Inserts values into an array,
 * skipping values that already exists.
 *
 * @param array Array to insert into.
 * @param values Values to insert.
 *
 * @example
 * const numbers = []
 * insertIntoArray(numbers, 1, 2, 3, 4, 5)
 * // numbers = [1, 2, 3, 4, 5]
 */
export declare function insertIntoArray<T>(array: T[], ...values: T[]): void;
/**
 * Removes values from an array.
 *
 * @param array Array to remove from.
 * @param values Values to remove.
 *
 * @example
 * const numbers = [0, 1, 2, 3, 4, 5]
 * removeFromArray(numbers, 0, 2, 4)
 * // numbers = [1, 3, 5]
 */
export declare function removeFromArray<T>(array: T[], ...values: T[]): void;
/**
 * Removes all values from an entity.
 * Does not remove the key value.
 *
 * @param entity Entity to clear.
 *
 * @example
 * clearEntity(people)
 */
export declare function clearEntity(entity: Entity): void;
/**
 * Copies an entity with all its values.
 * The values themselves are not copied,
 * so they keep their referential equality.
 *
 * @param entity Entity to copy.
 *
 * @example
 * copyEntity(people)
 */
export declare function copyEntity<T>(entity: Entity<T>): Entity<T, { [K in keyof T]: T[K] extends string | number | symbol ? K : never; }[keyof T]>;
/**
 * Inserts values into an entity.
 *
 * @param entity Entity to insert into.
 * @param values Values to insert.
 *
 * @example
 * insertIntoEntity(people, matt, sam, liam)
 */
export declare function insertIntoEntity<E extends Entity>(entity: E, ...values: ValueType<E>[]): void;
/**
 * Removes values from an entity.
 *
 * @param entity Entity to remove from.
 * @param values Values to remove.
 *
 * @example
 * removeFromEntity(people, matt, sam, liam)
 */
export declare function removeFromEntity<E extends Entity>(entity: E, ...values: ValueType<E>[]): void;
/**
 * Removes values from an entity by matching IDs.
 *
 * @param entity Entity to remove from.
 * @param ids IDs to match.
 *
 * @example
 * removeFromEntityById(people, 0, 2, 6)
 */
export declare function removeFromEntityById<E extends Entity>(entity: E, ...ids: KeyType<E>[]): void;
